package com.miaxis.common.utils.wx;

import com.miaxis.common.utils.XmlUtil;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.StringWriter;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 微信消息工具
 * @author wwl
 * @version 1.0
 * @date 2021/10/25 13:19
 */
public class MessageUtil {

    public static final String MESSAGE_TEXT="text";//文本消息
    public static final String MESSAGE_IMAGE="image";//图片消息
    public static final String MESSAGE_VOICE="voice";//语音消息
    public static final String MESSAGE_VIDEO="video";//视频消息
    public static final String MESSAGE_LOCATION="LOCATION";//地理位置消息
    public static final String MESSAGE_EVENT="event";//事件消息
    public static final String MESSAGE_SUBSCRIBE="subscribe";//关注
    public static final String MESSAGE_UNSUBSCRIBE="unsubscribe";//取消关注
    public static final String MESSAGE_SCAN="SCAN";//已关注扫码消息
    public static final String MESSAGE_CLICK="CLICK";//点击菜单拉取消息
    public static final String MESSAGE_VIEW="VIEW";//点击菜单跳转链接消息
    public static final String MESSAGE_VIEW_MINIPROGRAM="view_miniprogram";//点击菜单跳转小程序的消息



    /**
     * 回复文本消息
     * @param toUserName
     * @param fromUserName
     * @param content
     * @return
     */
    public static String initText(String toUserName, String fromUserName, String content) throws Exception{
        // 返回消息时ToUserName的值与FromUserName的互换
        Map<String, String> returnMap = new HashMap<>();
        returnMap.put("ToUserName", fromUserName);
        returnMap.put("FromUserName", toUserName);
        returnMap.put("CreateTime", new Date().getTime()+"");
        returnMap.put("MsgType", "text");
        returnMap.put("Content", content);

        return mapToXml(returnMap);
    }


    /**
     * 回复图文消息
     * @param toUserName 接收方帐号（收到的OpenID）
     * @param fromUserName 开发者微信号
     * @param articles 图文消息信息，注意，如果图文数超过限制，则将只发限制内的条数
     * @param title 图文消息标题
     * @param description 图文消息描述
     * @param picUrl 图片链接，支持JPG、PNG格式，较好的效果为大图360*200，小图200*200
     * @param url 点击图文消息跳转链接
     * @return
     */
    public static String initNews(String toUserName, String fromUserName, String articles, String title, String description, String picUrl, String url)  throws Exception{
        // 返回消息时ToUserName的值与FromUserName的互换
        Map<String, String> returnMap = new HashMap<>();
        returnMap.put("ToUserName", fromUserName);
        returnMap.put("FromUserName", toUserName);
        returnMap.put("CreateTime", new Date().getTime()+"");
        returnMap.put("MsgType", "news");
        returnMap.put("ArticleCount", "1");
        returnMap.put("Content", articles);
        returnMap.put("Title", title);
        returnMap.put("Description", description);
        returnMap.put("PicUrl", picUrl);
        returnMap.put("Url", url);
        return mapToXml(returnMap);
    }


    /**
     * 将Map转换为XML格式的字符串
     *
     * @param data Map类型数据
     * @return XML格式的字符串
     * @throws Exception
     */
    public static String mapToXml(Map<String, String> data) throws Exception {

        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder= documentBuilderFactory.newDocumentBuilder();
        org.w3c.dom.Document document = documentBuilder.newDocument();
        org.w3c.dom.Element root = document.createElement("xml");
        document.appendChild(root);
        for (String key: data.keySet()) {
            String value = data.get(key);
            if (value == null) {
                value = "";
            }
            value = value.trim();
            org.w3c.dom.Element filed = document.createElement(key);
            filed.appendChild(document.createTextNode(value));
            root.appendChild(filed);
        }
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer transformer = tf.newTransformer();
        DOMSource source = new DOMSource(document);
        transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        StringWriter writer = new StringWriter();
        StreamResult result = new StreamResult(writer);
        transformer.transform(source, result);
        String output = writer.getBuffer().toString(); //.replaceAll("\n|\r", "");
        try {
            writer.close();
        } catch (Exception ex) {

        }
        return output;
    }


}
